
import json

class ObjectJSON(object):

    def toJSON(self):

        dicionary = json.loads(json.dumps(self, default=lambda o: o.__dict__))

        dicionary = remove_none(dicionary)

        dicionary = process_name_key(dicionary)

        return json.dumps(dicionary, indent=2)

    def prepare(self):
        pass

    def __getattribute__(self, attribute):

        if attribute == '__dict__':
            self.prepare()

        return object.__getattribute__(self, attribute)


def process_name_key(dictionary):

    if not isinstance(dictionary, dict):
        return dictionary

    newDictionary = {}

    for key in dictionary:
        newDictionary[capitalize_key(key)] = process_name_key(dictionary[key])

    return newDictionary

def capitalize_key(key):
    parts = key.split('_')

    newParts = []
    for part in parts:
        newParts.append(part.capitalize())

    return ''.join(newParts)

def remove_none(dado):

    if isinstance(dado, dict):
        return remove_none_dict(dado)
    elif isinstance(dado, list):
        return remove_none_list(dado)

    return dado

def remove_none_dict(obj):
    retorno = {}
    for chave in obj:
        valor = obj[chave]
        if valor or isinstance(valor, (int, int, float, complex)):
            retorno[chave] = remove_none(valor)

    return retorno

def remove_none_list(lista):
    resposta = []
    for linha in lista:
        valor = remove_none(linha)
        resposta.append(valor)

    return resposta




class Customer(object):

    def __init__(self, name):

        self.name = name
        self.email = None
        self.birth_date = None
        self.identity = None
        self.identity_type = None
        self.address = None
        self.delivery_adress = None




class Address(object):

    def __init__(self):

        self.street = None
        self.number = None
        self.complement = None
        self.zip_code = None
        self.city = None
        self.state = None
        self.country = None




from .base import Base

class UpdateSale(Base):

    def __init__(self, type, merchant, environment):

        super(UpdateSale, self).__init__(merchant)

        self.environment = environment
        self.type = type
        self.service_tax_amount = None
        self.amount = None

    def execute(self, payment_id):

        uri = '%s1/sales/%s/%s' % (self.environment.api, payment_id, self.type)

        params = {}

        if self.amount:
            params['amount'] = self.amount

        if self.service_tax_amount:
            params['serviceTaxAmount'] = self.service_tax_amount

        return self.send_request('PUT', uri, params=params)




from .objectJSON import ObjectJSON

class Sale(ObjectJSON):

    def __init__(self, merchant_order_id):

        self.merchant_order_id = merchant_order_id
        self.customer = None
        self.payment = None
        self.debitcard = None

    def update_return(self, r):

        payment = r.get('Payment') or {}
        debitcard = payment.get('DebitCard') or {}
        self.payment.payment_id = payment.get('PaymentId')
        self.payment.url = payment.get('Url')
        self.payment.authentication_url = payment.get('AuthenticationUrl') or {}
        self.payment.tid = payment.get('Tid')
        self.payment.status = payment.get('Status')
        self.payment.return_code = payment.get('ReturnCode')
        self.payment.proof_of_sale = payment.get('ProofOfSale')
        self.payment.debit_card.card_number = debitcard.get('CardNumber') or {}

        if self.payment.recurrent_payment:
            recurrent = payment.get('RecurrentPayment') or {}
            self.payment.recurrent_payment.recurrent_payment_id = recurrent.get('RecurrentPaymentId')



class Environment(object):

    def __init__(self, sandbox):

        # Production
        if not sandbox:
            self.api = 'https://api.cieloecommerce.cielo.com.br/'
            self.api_query = 'https://apiquery.cieloecommerce.cielo.com.br/'
        else:
            self.api = 'https://apisandbox.cieloecommerce.cielo.com.br/'
            self.api_query = 'https://apiquerysandbox.cieloecommerce.cielo.com.br/'




from .request.createSale import CreateSale
from .request.querySale import QuerySale
from .request.updateSale import UpdateSale
from .request.createCardToken import CreateCardToken
from .request.queryRecorrency import QueryRecorrency
from .request.deactivateRecorrency import DeactivateRecorrency
from .request.reactivateRecorrency import ReactivateRecorrency

class CieloEcommerce(object):

    def __init__(self, merchant, environment):

        self.environment = environment
        self.merchant = merchant

    def create_sale(self, sale):

        request = CreateSale(self.merchant, self.environment)

        return request.execute(sale)

    def capture_sale(self, payment_id, amount = None, service_tax_amount = None):
        request = UpdateSale('capture', self.merchant, self.environment)

        request.amount = amount
        request.service_tax_amount = service_tax_amount

        return request.execute(payment_id)

    def cancel_sale(self, payment_id, amount = None):
        request = UpdateSale('void', self.merchant, self.environment)

        request.amount = amount

        return request.execute(payment_id)

    def get_sale(self, payment_id):
        request = QuerySale(self.merchant, self.environment)

        return request.execute(payment_id)

    def create_card_token(self, creditCard):

        request = CreateCardToken(self.merchant, self.environment)

        return request.execute(creditCard)

    def get_recurrent_payment(self, recurrent_payment_id):

        request = QueryRecorrency(self.merchant, self.environment)

        return request.execute(recurrent_payment_id)

    def deactivate_recurrent_payment(self, recurrent_payment_id):

        request = DeactivateRecorrency(self.merchant, self.environment)

        return request.execute(recurrent_payment_id)

    def reactivate_recurrent_payment(self, recurrent_payment_id):

        request = ReactivateRecorrency(self.merchant, self.environment)

        return request.execute(recurrent_payment_id)





from setuptools import setup, find_packages

setup(
    name='lins_cieloApi3',
    version='0.1.0', 
    description='Pacote customizado da sdk python da cielo, https://github.com/DeveloperCielo/API-3.0-Python',
    url='https://lpsvn.lojaspompeia.com.br/svn/pypck-lins_cieloApi3/',
    author='Halyson Sampaio',
    author_email='halyson@lojaspompeia.com.br',
    packages=find_packages(),
    zip_safe=False
)



INTERVAL_MONTHLY = "Monthly"
INTERVAL_BIMONTHLY = "Bimonthly"
INTERVAL_QUARTERLY = "Quarterly"
INTERVAL_SEMIANNUAL = "SemiAnnual"
INTERVAL_ANNUAL = "Annual"

class RecurrentPayment(object):

    def __init__(self, authorize_now=True):

        self.authorize_now = authorize_now
        self.start_date = None
        self.end_date = None
        self.interval = None
        self.recurrent_payment_id = None
